// Copyright (c) FIRST and other WPILib contributors.
// Open Source Software; you can modify and/or share it under the terms of
// the WPILib BSD license file in the root directory of this project.

package frc.robot.subsystems;

import java.util.Map;

import edu.wpi.first.wpilibj.AddressableLED;
import edu.wpi.first.wpilibj.AddressableLEDBuffer;
import edu.wpi.first.wpilibj.shuffleboard.BuiltInLayouts;
import edu.wpi.first.wpilibj.shuffleboard.Shuffleboard;
import edu.wpi.first.wpilibj.shuffleboard.ShuffleboardLayout;
import edu.wpi.first.wpilibj.shuffleboard.ShuffleboardTab;
import edu.wpi.first.wpilibj2.command.RunCommand;
import edu.wpi.first.wpilibj2.command.SubsystemBase;

import static frc.robot.Constants.*;

public class Led extends SubsystemBase {

  private final ShuffleboardTab tab = Shuffleboard.getTab("Peanut");
  private final ShuffleboardLayout layout = tab.getLayout("LED", BuiltInLayouts.kList)
    .withSize(2, 2)
    .withProperties(Map.of("Label position", "HIDDEN"));
  
  private final AddressableLED led = new AddressableLED(LED_PORT);
  private final AddressableLEDBuffer buffer = new AddressableLEDBuffer(150);

  private int rainbowFirstPixelHue = 0;
  private int cycleHue = 0;
  private int chaseFirstPixel = 0;

  /** Creates a new Led. */
  public Led() {
    led.setLength(buffer.getLength());
    led.setData(buffer);
    led.start();

    layout.add(new RunCommand(() -> rainbow(), this).withName("Rainbow"));
    layout.add(new RunCommand(() -> colorCycle(), this).withName("Color Cycle"));
    layout.add(new RunCommand(() -> chase(), this).withName("Chase"));
  }

  public void rainbow() {
    for (int i = 0; i < buffer.getLength(); i++) {
      final int hue = (rainbowFirstPixelHue + (i * 180 / buffer.getLength())) % 180;
      buffer.setHSV(i, hue, 255, 128);
    }
    rainbowFirstPixelHue += 3;
    rainbowFirstPixelHue %= 180;
  }

  public void colorCycle() {
    for (int i = 0; i < buffer.getLength(); i++) {
      buffer.setHSV(i, cycleHue, 255, 128);
    }
    cycleHue += 1;
    cycleHue %= 180;
  }

  public void chase() {
    int lastPixel = (chaseFirstPixel + buffer.getLength()/15) % buffer.getLength();
    if (lastPixel < chaseFirstPixel) {
      for (int i = 0; i < buffer.getLength(); i++) {
        if (i < lastPixel || i >= chaseFirstPixel) {
          buffer.setHSV(i, 180, 255, 255);
        } else {
          buffer.setHSV(i, 180, 0, 0);
        }
      }
    } else {
      for (int i = 0; i < buffer.getLength(); i++) {
        if (i >= chaseFirstPixel && i < lastPixel) {
          buffer.setHSV(i, 180, 255, 255);
        } else {
          buffer.setHSV(i, 180, 0, 0);
        }
      }
    }
    chaseFirstPixel += 1;
    chaseFirstPixel %= buffer.getLength();
  }

  @Override
  public void periodic() {
    // This method will be called once per scheduler run
    led.setData(buffer);
  }
}
